def extract_title(markdown):
    for line in markdown.split('\n'):
        if ".title[" in line:
            start = line.index(".title[") + len(".title[")
            end = line.rindex("]", start)
            return line[start:end]
        if '#' in line:
            index_first_non_whitespace = len(line) - len(line.lstrip())
            index_first_hash = line.index('#')
            first_alpha = list(filter(str.isalpha, str(line)))[0]
            index_first_alpha = line.index(first_alpha)
            if index_first_non_whitespace == index_first_hash:
                return line[index_first_alpha:]
    # we found nothing, we default to this
    return 'cicero.xyz'


def test_extract_title():
    markdown = '''name: inverse
    layout: true
    class: center, middle, inverse

    ---

    # Talking to C/C++/Fortran via CFFI

    ## Radovan Bast

    High Performance Computing Group,
    UiT The Arctic University of Norway

    Licensed under [CC BY 4.0](https://creativecommons.org/licenses/by/4.0/).
    Code examples: [OSI](http://opensource.org)-approved [MIT license](http://opensource.org/licenses/mit-license.html).

    ---'''
    markdown_alt = '''name: inverse
    layout: true
    class: center, middle, inverse

    ---

    .title[Talking to [C/C++/Fortran] via CFFI]

    .author[Radovan Bast]

    High Performance Computing Group,
    UiT The Arctic University of Norway

    Licensed under [CC BY 4.0](https://creativecommons.org/licenses/by/4.0/).
    Code examples: [OSI](http://opensource.org)-approved [MIT license](http://opensource.org/licenses/mit-license.html).

    ---'''
    assert extract_title(markdown) == 'Talking to C/C++/Fortran via CFFI'
    assert extract_title(markdown_alt) == 'Talking to [C/C++/Fortran] via CFFI'



def expand_img_link(s, prefix):
    import re

    if '![' in s and '](' in s:
        if 'http' not in s:
            p = re.compile(r'!\[(.*)\]\(')
            alt = p.findall(s)[0]
            p = re.compile(r'!\[.*\]\(([^")]+)')
            src = p.findall(s)[0]
            return "![{0}]({1}{2})\n".format(alt, prefix, src)

    if '<img' in s and 'src="' in s:
        if 'http' not in s:
            p = re.compile(r'<img [^>]*src="([^"]+)')
            src = p.findall(s)[0]
            return s.replace(src, prefix + src)

    if 'background-image: url(' in s:
            p = re.compile(r'background-image: url\(([^")]+)')
            src = p.findall(s)[0]
            return s.replace(src, prefix + src)

    return s


def test_expand_img_link():
    assert expand_img_link('![Raboof](img/pie.jpg)', 'foo/') == '![Raboof](foo/img/pie.jpg)\n'
    assert expand_img_link('![Foo oof 123](img/pie.jpg)', 'http://') == '![Foo oof 123](http://img/pie.jpg)\n'
    assert expand_img_link('[Raboof](img/pie.jpg)', 'foo/') == '[Raboof](img/pie.jpg)'
    assert expand_img_link('img src', 'foo/') == 'img src'
    assert expand_img_link('<img src="img/phd_final.gif" style="width: 400px;"/>', 'foo/') == '<img src="foo/img/phd_final.gif" style="width: 400px;"/>'
    assert expand_img_link('background-image: url(foo.png)', 'bar/') == 'background-image: url(bar/foo.png)'


def fix_images(s, prefix):
    return '\n'.join([expand_img_link(line, prefix) for line in s.split('\n')])



import os
import flask
import sys
import requests
import json

blueprint = flask.Blueprint('git', __name__)

URL_BASE = 'CICERO_URL_BASE_is_undefined'
_url_base = os.environ.get('CICERO_URL_BASE')
if _url_base is not None:
    URL_BASE = _url_base


def get_sha_github(owner, repo, ref):
    uri = 'https://api.github.com/repos/{0}/{1}/commits/{2}'.format(owner, repo, ref)
    response = requests.get(uri)
    data = json.loads(response.text)
    return data['sha']


def test_get_sha_github():
    is_travis_build = os.getenv('TRAVIS', False)
    # Travis CI is rate-limited by GitHub API
    if not is_travis_build:
        sha = get_sha_github('bast', 'cicero', 'bfa3748447')
        assert sha == 'bfa3748447fe0c7455f19a027575406a0c561a4f'


def set_url_base(host, port):
    global URL_BASE
    URL_BASE = 'http://{}:{}'.format(host, port)


@blueprint.route('/')
def home():
    return flask.render_template('index.html', url_base=URL_BASE)


def render_url_markdown(path, engine, engine_version):
    from .title import extract_title
    from .images import fix_images

    service, owner, repo, ref, *_md_file_path = path.split('/')
    md_file_path = '/'.join(_md_file_path)
    md_file_path_root = '/'.join(_md_file_path[:-1])
    md_file = _md_file_path[-1]

    if service == 'github.com':
        # we need to translate the reference to a sha (the reference can be a sha)
        # the reason for this is that cdn.rawgit.com caches files forever
        # the reference may change but the sha won't
        # when changing to https://www.jsdelivr.com i haven't verified whether
        # results are cached so translating to hashes might be overkill
        sha = get_sha_github(owner, repo, ref)

        root = '{0}/{1}@{2}'.format(owner, repo, sha)

        url_prefix = 'https://cdn.jsdelivr.net/gh/{0}/'.format(root)
    elif service == 'gitlab.com':
        url_prefix = 'https://{0}/{1}/{2}/raw/{3}/'.format(service, owner, repo, ref)
    else:
        *_url_prefix, md_file = path.split('/')
        url_prefix = 'https://' + '/'.join(_url_prefix)
        md_file_path = md_file
        md_file_path_root = ''

    md_file_prefix, _ = os.path.splitext(md_file)

    if '/' in md_file_path:
        url_prefix += md_file_path_root

    url = url_prefix + '/' + md_file

    response = requests.get(url)
    if response.status_code == 404:
        return flask.render_template('404.html')

    markdown = response.text

    title = extract_title(markdown)
    style = flask.request.args.get('style')
    if style is None:
        style = 'default'

    # if own css is available, we load it
    # if not, we default to empty own css
    url = url_prefix + '/' + md_file_prefix + '.css'
    response = requests.get(url)
    if response.status_code == 404:
        own_css = ''
    else:
        own_css = response.text
        own_css = flask.Markup(own_css)  # disable autoescaping

    # .. do the same for own javascript
#   url = url_prefix + '/' + md_file_prefix + '.js'
#   response = requests.get(url)
#   if response.status_code == 404:
#       own_javascript = ''
#   else:
#       own_javascript = response.text
    # for the moment I am not sure whether the above is not too risky
    own_javascript = ''

    # use custom configuration for the rendering engine, if available
    url = url_prefix + '/' + md_file_prefix + '.conf'
    response = requests.get(url)
    if response.status_code == 404:
        own_conf = ''
    else:
        own_conf = ',\n'.join(response.text.split('\n'))

    return flask.render_template('render.html',
                                 title=title,
                                 markdown=fix_images(markdown, url_prefix),
                                 style=style,
                                 own_css=own_css,
                                 own_javascript=own_javascript,
                                 own_conf=own_conf,
                                 engine='{0}-{1}'.format(engine, engine_version))


@blueprint.route('/v1/github/<path:path>/remark/')
def render_v1(path):
    return render_url_markdown('github.com' + '/' + path, 'remark', 'legacy')


@blueprint.route('/v2/remark/github/<path:path>/')
def render_v2(path):
    return render_url_markdown('github.com' + '/' + path, 'remark', 'legacy')


@blueprint.route('/v3/<string:engine>/<string:engine_version>/<path:path>/')
def render_v3(path, engine, engine_version):
    return render_url_markdown(path, engine, engine_version)



from flask import Blueprint

blueprint = Blueprint('preview', __name__)


@blueprint.route('/')
def home():
    import io
    import os
    from flask import current_app, render_template, render_template_string, request, Markup
    from .title import extract_title
    from .images import fix_images

    config = current_app.config

    try:
        with io.open(config['filename'], 'r', encoding='utf-8') as mkdfile:
            markdown = mkdfile.read()
    except UnicodeDecodeError:
        with io.open(config['filename'], 'r', encoding='cp1252') as mkdfile:
            markdown = mkdfile.read()

    title = extract_title(markdown)
    markdown = fix_images(markdown, 'images/')

    style = request.args.get('style')
    if style is None:
        style = 'default'

    talk_no_suffix, _suffix = os.path.splitext(config['filename'])
    own_css_file = talk_no_suffix + '.css'
    own_css = ''  # by default no own css
    if os.path.isfile(own_css_file):
        with io.open(own_css_file, 'r') as css_file:
            own_css = css_file.read()
    own_css = Markup(own_css)  # disable autoescaping
    # use own javascript, if available
    own_js_file = talk_no_suffix + '.js'
    own_javascript = ''
    if os.path.isfile(own_js_file):
        with io.open(own_js_file, 'r') as js_file:
            own_javascript = js_file.read()
    # use custom configuration for the rendering engine, if available
    own_conf_file = talk_no_suffix + '.conf'
    own_conf = ''
    if os.path.isfile(own_conf_file):
        with io.open(own_conf_file, 'r') as conf_file:
            for line in conf_file.readlines():
                own_conf += line.replace('\n', ',\n')
            own_conf = own_conf.rstrip('\n')

    return render_template('render.html',
                           title=title,
                           markdown=markdown,
                           style=style,
                           own_css=own_css,
                           own_javascript=own_javascript,
                           own_conf=own_conf,
                           engine=config['engine'])


@blueprint.route('/images/<path:path>')
def serve_image(path):
    from flask import send_from_directory, current_app
    config = current_app.config
    return send_from_directory(config['imagedir'], path)


@blueprint.errorhandler(404)
def page_not_found(e):
    from flask import render_template
    return render_template('404.html'), 404



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import os


def parse_args():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    arg = parser.add_argument

    arg('--file', '-f', dest='filename', help='serve a local file')
    arg('--engine', dest='engine', help='rendering engine (for talks pre-v3 use remark-legacy)', default='remark-0.13.0')
    arg('--debug', dest='debug', action='store_true', default=False)
    arg('--host', dest='host', default=os.environ.get('HOST', '127.0.0.1'))
    arg('--port', dest='port', type=int, default=int(os.environ.get('PORT', 5000)))

    return parser.parse_args()



from flask import Flask, render_template


def _get_subdir(dirname):
    import os
    return os.path.join(os.path.dirname(__file__), dirname)


app = Flask('Cicero',
            template_folder=_get_subdir('templates'),
            static_folder=_get_subdir('static'))


@app.errorhandler(404)
def page_not_found(e):
    return render_template('404.html'), 404



import os
import sys
from .app import app
from . import git
from . import preview
from .cli import parse_args


def main():
    args = parse_args()

    if args.filename:

        if not os.path.isfile(args.filename):
            sys.stderr.write('ERROR: file {0} not found\n'.format(args.filename))
            sys.exit(1)

        app.config['filename'] = args.filename
        app.config['engine'] = args.engine
        app.config['imagedir'] = os.path.dirname(args.filename)
        app.register_blueprint(preview.blueprint)
    else:
        git.set_url_base(args.host, args.port)
        app.register_blueprint(git.blueprint)

    app.debug = args.debug

    app.run(host=args.host, port=args.port)



from setuptools import setup
import os
import sys

_here = os.path.abspath(os.path.dirname(__file__))

if sys.version_info[0] < 3:
    with open(os.path.join(_here, 'README.rst')) as f:
        long_description = f.read()
else:
    with open(os.path.join(_here, 'README.rst'), encoding='utf-8') as f:
        long_description = f.read()

version = {}
with open(os.path.join(_here, 'cicero', 'version.py')) as f:
    exec(f.read(), version)

def package_files(directory):
    paths = []
    for (path, directories, filenames) in os.walk(directory, followlinks=True):
        for filename in filenames:
            paths.append(os.path.join('..', path, filename))
    return paths

extra_files = package_files('cicero/static') + \
              package_files('cicero/templates')

setup(
    name='cicero',
    version=version['__version__'],
    description='Cicero - Serving presentation slides written in Markdown.',
    long_description=long_description,
    author='Radovan Bast',
    author_email='bast@users.noreply.github.com',
    url='https://github.com/bast/cicero',
    license='GNU Affero General Public License v3',
    packages=['cicero'],
    package_data={'': extra_files},
    entry_points={'console_scripts': ['cicero = cicero.main:main']},
    install_requires=[
        'requests==2.19.1',
        'flask==1.0.2'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Science/Research',
        'Programming Language :: Python :: 3.6'
    ],
)
